from typing import Optional, Type, Callable

from cpl_core.application import StartupABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.environment import ApplicationEnvironment

from bot_core.abc.custom_file_logger_abc import CustomFileLoggerABC
from bot_core.configuration.bot_logging_settings import BotLoggingSettings
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.logging.database_logger import DatabaseLogger
from migration_to_sql.mock.migration_service import MigrationService


class Startup(StartupABC):
    def __init__(self):
        StartupABC.__init__(self)

    def configure_configuration(
        self, configuration: ConfigurationABC, environment: ApplicationEnvironment
    ) -> ConfigurationABC:
        configuration.add_json_file("appsettings.json")
        configuration.add_configuration(FeatureFlagsSettings, FeatureFlagsSettings())
        self._configure_settings_with_sub_settings(
            configuration, BotLoggingSettings, lambda x: x.files, lambda x: x.key
        )
        return configuration

    def configure_services(
        self, services: ServiceCollectionABC, environment: ApplicationEnvironment
    ) -> ServiceProviderABC:
        services.add_logging()
        services.add_singleton(CustomFileLoggerABC, DatabaseLogger)
        services.add_transient(MigrationService)
        return services.build_service_provider()

    @staticmethod
    def _configure_settings_with_sub_settings(
        config: ConfigurationABC, settings_type: Type, list_atr: Callable, atr: Callable
    ):
        settings: Optional[settings_type] = config.get_configuration(settings_type)
        if settings is None:
            return

        for sub_settings in list_atr(settings):
            config.add_configuration(f"{type(sub_settings).__name__}_{atr(sub_settings)}", sub_settings)
